/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package com.huawei.agc.flutter.cloudfunctions;

import androidx.annotation.NonNull;

import com.huawei.agconnect.function.AGConnectFunction;
import com.huawei.agconnect.function.FunctionCallable;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

final class CloudFunctionsMethodCallHandler implements MethodCallHandler {
    private final AGConnectFunction agConnectFunction;

    CloudFunctionsMethodCallHandler(@NonNull final AGConnectFunction agConnectFunction) {
        this.agConnectFunction = agConnectFunction;
    }

    private void callFunction(@NonNull final MethodCall methodCall, @NonNull final Result result) {
        final FunctionCallable functionCallable = FunctionUtils.getFunctionCallable(methodCall, agConnectFunction);
        final Object functionParameters = methodCall.argument("functionParameters");

        functionCallable.call(functionParameters)
                .addOnSuccessListener(functionResult -> FunctionUtils.sendSuccess(result, functionResult))
                .addOnFailureListener(e -> FunctionUtils.sendError(result, e));
    }

    @Override
    public void onMethodCall(@NonNull final MethodCall methodCall, @NonNull final Result result) {
        if (methodCall.method.equals("callFunction")) {
            callFunction(methodCall, result);
        } else {
            result.notImplemented();
        }
    }
}
