/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

// Handling blueprint and declerations.
protocol AGCCloudFunctionsHanding {
    func handle(resolve: @escaping FlutterResult, _ isSuccess: Bool?, _ error: Error?)
    func handle<T>(resolve: FlutterResult, _ instance: T)
    func fail(with error: Error, resolve: @escaping FlutterResult)
}

extension AGCCloudFunctionsHanding {
    func handle(resolve: @escaping FlutterResult, _ isSuccess: Bool?=true, _ error: Error?=nil) {
        Result.shared.resolve(resolve: resolve, isSuccess, error: error)
    }
    
    func handle<T>(resolve: FlutterResult, _ instance: T) {
        Result.shared.resolve(resolve: resolve, instance: instance)
    }
    
    func fail(with error: Error, resolve: @escaping FlutterResult){
        Result.shared.resolve(resolve: resolve, false, error: error)
    }
}

class Result {
    static let shared = Result()
    
    private init() { }
    
    func resolve<T>(resolve: FlutterResult, instance: T?) {
        resolve(instance)
    }
    
    func resolve(resolve: @escaping FlutterResult, _ isSuccess: Bool?, error: Error?) {
        guard let success = isSuccess else { return }
        if success {
            resolve(success)
        }else {
            resolve(FlutterError(code: "-1", message: error.debugDescription , details: error?.localizedDescription))
        }
    }
}
