/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';

import 'agc_timeunit.dart';
import 'function_result.dart';

class FunctionCallable {
  static const int DEFAULT_TIMEOUT = 70;
  static const AGCTimeUnit DEFAULT_TIMEUNIT = AGCTimeUnit.SECONDS;

  String httpTriggerURI;
  int timeout;
  AGCTimeUnit units;

  FunctionCallable(
    this.httpTriggerURI, {
    this.timeout = 70,
    this.units = AGCTimeUnit.SECONDS,
  });

  factory FunctionCallable.fromJson(String source) {
    return FunctionCallable.fromMap(json.decode(source));
  }

  factory FunctionCallable.fromMap(Map<String, dynamic> map) {
    return FunctionCallable(
      map['httpTriggerURI'],
      timeout: map['timeout'],
      units: AGCTimeUnit.fromInt(map['units']),
    );
  }

  Future<FunctionResult> call([dynamic parameters]) async {
    final FunctionResult functionResult =
        await _AGConnectCloudFunctions.callFunction(
      arguments: toMap()..putIfAbsent('functionParameters', () => parameters),
    );
    return functionResult;
  }

  FunctionCallable clone(
      {String? httpTriggerURI, int? timeout, AGCTimeUnit? units}) {
    return FunctionCallable(
      httpTriggerURI ?? this.httpTriggerURI,
      timeout: timeout ?? this.timeout,
      units: units ?? this.units,
    );
  }

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'httpTriggerURI': httpTriggerURI,
      'timeout': timeout,
      'units': units.toInt(),
    };
  }

  String toJson() => json.encode(toMap());

  @override
  String toString() =>
      'FunctionCallable(httpTriggerURI: $httpTriggerURI, timeout: $timeout, units: $units)';

  @override
  bool operator ==(Object other) {
    return identical(this, other) ||
        (other is FunctionCallable &&
            other.httpTriggerURI == httpTriggerURI &&
            other.timeout == timeout &&
            other.units == units);
  }

  @override
  int get hashCode =>
      httpTriggerURI.hashCode ^ timeout.hashCode ^ units.hashCode;
}

class _AGConnectCloudFunctions {
  static const MethodChannel _methodChannel = const MethodChannel(
      'com.huawei.agc.flutter.cloudfunctions/MethodChannel');

  static Future<FunctionResult> callFunction(
      {Map<String, dynamic> arguments = const <String, dynamic>{}}) async {
    final String? result =
        await _methodChannel.invokeMethod<String>('callFunction', arguments);
    return FunctionResult(result!);
  }
}
