/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

@immutable
class FunctionError {
  static const int UNKNOW_ERROR_CODE = -1;

  final String message;
  final int code;

  FunctionError(this.message, this.code);

  factory FunctionError.fromException(PlatformException e) {
    return FunctionError(e.message ?? '', int.parse(e.code));
  }

  factory FunctionError.fromJson(String source) {
    return FunctionError.fromMap(json.decode(source));
  }

  factory FunctionError.fromMap(Map<String, dynamic> map) {
    return FunctionError(
      map['message'] ?? '',
      map['code'] ?? UNKNOW_ERROR_CODE,
    );
  }

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'message': message,
      'code': code,
    };
  }

  String toJson() => json.encode(toMap());

  @override
  String toString() => 'FunctionError(message: $message, code: $code)';

  @override
  bool operator ==(Object other) {
    return identical(this, other) ||
        (other is FunctionError &&
            other.message == message &&
            other.code == code);
  }

  @override
  int get hashCode => message.hashCode ^ code.hashCode;
}
